Attribute VB_Name = "modCrepusculos"
Option Explicit
 
Const SE_GREG_CAL As Integer = 1
Const PI = 3.141592

Private Declare Function swe_julday Lib "swedll32.dll" _
        Alias "_swe_julday@24" _
( _
   ByVal Year As Long, _
   ByVal Month As Long, _
   ByVal Day As Long, _
   ByVal hour As Double, _
   ByVal gregflg As Long _
) As Double

'Dado el ao, mes, y da; y la latitud y longitud de
'un lugar, calcula los incrementos respecto de la hora
'de ese lugar (en GMT) para los crepsculos civil, nutico
'y astronmico.

Public Sub CalcularCrepusculos( _
    Y As Integer, MM As Integer, D As Integer, _
    Latitud As Double, Longitud As Double, _
    H As Double, WE As String, NS As String, _
    UTCivM As Double, UTNauM As Double, UTAstM As Double, _
    UTCivE As Double, UTNauE As Double, UTAstE As Double)
    
    Dim L As Double, fi As Double, JD_0 As Double, JD_t As Double
    Dim N As Integer, tm As Double, te As Double
    
    'Clculos

    L = Abs(Longitud / 15)
    fi = Abs(Latitud)
        
    If (WE = "E") Then
        L = -L
    End If
        
    If (NS = "S") Then
        fi = -fi
    End If
    
    'Fecha correspondiente al calendario juliano de la introducida.
    JD_t = swe_julday(Y, MM, D, 0#, SE_GREG_CAL)
    'Fecha correspondiente al calendario juliano del 0 de enero a las 0 horas.
    JD_0 = swe_julday(Y, 1, 0, 0#, SE_GREG_CAL)
    
    N = Fix(JD_t - JD_0)
    tm = N + (6 + L) / 24
    te = N + (18 + L) / 24
    
    Call Tiempitos(1, tm, H, L, fi, UTCivM, UTNauM, UTAstM) 'Por la maana
    Call Tiempitos(2, te, H, L, fi, UTCivE, UTNauE, UTAstE) 'Por la tarde
End Sub

Private Sub Tiempitos(am_pm, tme, Alto, L, fi, UT1, UT2, UT3)
    Dim z(0 To 3) As Double, zrad(0 To 3) As Double
    Dim x(0 To 3) As Double, H(0 To 3) As Double, T(0 To 3) As Double
    Dim M As Double, Mrad As Double, i As Integer
    Dim landa As Double, landarad As Double
    Dim delta As Double, deltarad As Double
    Dim alfa As Double, firad As Double
    
    M = 0.9856 * tme - 3.289: Mrad = M * PI / 180#
    landa = M + 1.916 * Sin(Mrad) + 0.02 * Sin(2 * Mrad) + 282.634
    landa = Resto(landa, 360): landarad = landa * PI / 180#
    alfa = (180 / PI) * Angulo(Cos(landarad), 0.91746 * Sin(landarad)) / 15
    delta = (180 / PI) * Asin(0.39782 * Sin(landarad))
    
    deltarad = delta * PI / 180#: firad = fi * PI / 180#
    
    'Distancias zenitales para amanecer-atardecer y
    'crepsculos civil, nutico y astronmico.
    z(0) = 90 + (50 + 2.12 * Sqr(Alto)) / 60 'Rise-Set
    z(1) = 96 + 2.12 * Sqr(Alto) / 60        'Civil
    z(2) = 102 + 2.12 * Sqr(Alto) / 60       'Nautico
    z(3) = 108 + 2.12 * Sqr(Alto) / 60       'Astronomico
    
    For i = 0 To 3
       zrad(i) = z(i) * PI / 180#
    
       x(i) = (Cos(zrad(i)) - Sin(deltarad) * Sin(firad)) / (Cos(deltarad) * Cos(firad))
    
       If (am_pm = 1) Then
          H(i) = (360 - (180 / PI) * Acos(x(i))) / 15
       Else
          H(i) = 180 * Acos(x(i)) / (PI * 15)
       End If
    
       T(i) = H(i) + alfa - 0.06571 * tme - 6.622
    
       'Se convierte el tiempo T a un rango entre 0 y 24h
       T(i) = Resto(T(i), 24)

       If (T(i) < 0) Then T(i) = T(i) + 24
    Next i
    
    'Clculo del incremento en UTs del fenmeno. Si UT es mayor que 24h,
    'el fenmeno ocurre en el da siguiente, hora de Greenwich.
    'Si UT es negativo, el fenmeno ocurre en el da anterior, hora de Greenwich.
    UT1 = Abs(T(1) - T(0))
    UT2 = Abs(T(2) - T(0))
    UT3 = Abs(T(3) - T(0))
End Sub

Private Function Angulo(x, Y)
    If (x = 0) Then
        If (Y > 0) Then
            Angulo = 0.5 * PI
        Else
            Angulo = -0.5 * PI
        End If
    Else
        Angulo = Atn(Y / x)
    
        If (x < 0) Then
            Angulo = Angulo + PI
        ElseIf (Y < 0) Then
            Angulo = Angulo + 2 * PI
        End If
    End If
End Function

Private Function Resto(a, b)
    If (b = 0) Then
        Resto = a
    Else
        Resto = a - b * Int(a / b)
    End If
End Function
